package com.beifan.foxlibc.framework.spring.service.jwt;

import com.beifan.foxlibc.framework.utils.JvmCache;
import com.beifan.foxlibc.framework.utils.JwtUtils;
import com.beifan.foxlibc.framework.utils.collection.Maps;
import com.beifan.foxlibc.framework.utils.config.TokenAlgorithmDecl;
import com.beifan.foxlibc.framework.utils.generator.UniqueUtils;
import com.beifan.foxlibc.framework.utils.security.DoubleCoder;
import com.beifan.foxlibc.framework.utils.time.DateUtils;
import com.beifan.foxlibc.framework.utils.time.TimeUnits;
import io.jsonwebtoken.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

/**
 * @author lts
 * Create time 2022/4/27
 */
@Slf4j
@Service
public class AuthenticationServiceImplements implements AuthenticationService {

    private static final Map<String, Object> _JWT_HEADER
            = Maps.ofMap("alg", "HS256", "typ", "JWT");

    private TokenAlgorithmDecl currentTokenAlgorithmDecl;
    private final Map<String, TokenAlgorithmDecl> tokenAlgorithmDeclMap;

    private static final String _ENV = "env";
    private static final String EXPIRE_TIME = "expireTime";

    /**
     * 本地管理Token
     */
    private final JvmCache tokenCache = new JvmCache();

    /**
     * 加载配置文件内容
     */
    public AuthenticationServiceImplements() {
        tokenAlgorithmDeclMap = TokenAlgorithmDecl.loadTokenAlgorithmDecls("jwtini.xml");
    }

    @Override
    public void useEnv(String name) {
        currentTokenAlgorithmDecl = tokenAlgorithmDeclMap.get(name);
        /* 根据这个codedecl key去解码不同算法的token */
        _JWT_HEADER.put("alg", currentTokenAlgorithmDecl.getAlgorithm().name());
        _JWT_HEADER.put("uid", UniqueUtils.uuid());
        _JWT_HEADER.put(_ENV, name);
    }

    @Override
    public String getEnv(String token) {
        return JwtUtils.payload(token, "env");
    }

    /**
     * 获取当前JWT使用的算法
     */
    private SignatureAlgorithm getAlgorithm()
    {
        return SignatureAlgorithm.forName((String) _JWT_HEADER.get("alg"));
    }

    @Override
    public String createToken(Map<String, Object> claims) {
        Date expireDateTime = TimeUnits.SECONDS.add(new Date(), (int) currentTokenAlgorithmDecl.getExpireTime());

        /* 添加 token 过期时间 */
        claims.put(EXPIRE_TIME, DateUtils.format(expireDateTime));
        System.out.println("put expire time: " + DateUtils.format(expireDateTime));

        JwtBuilder builder = Jwts.builder().setHeader(_JWT_HEADER)
                .addClaims(claims)
                .setExpiration(expireDateTime);

        /* 使用不同的算法生成token */
        SignatureAlgorithm algorithm = getAlgorithm();
        if (algorithm == SignatureAlgorithm.RS256)
            builder.signWith(algorithm, DoubleCoder.PRIVATERSA256(currentTokenAlgorithmDecl.getPrivateKey()));

        else if (algorithm == SignatureAlgorithm.ES256)
            builder.signWith(algorithm, DoubleCoder.PRIVATEEC256(currentTokenAlgorithmDecl.getPrivateKey()));

        else if (algorithm == SignatureAlgorithm.HS256)
            builder.signWith(algorithm, currentTokenAlgorithmDecl.getSecret());

        String token = builder.compact();
        /* 将token存入缓存 */
        tokenCache.set(token, claims, expireDateTime);

        return token;
    }

    @Override
    @SuppressWarnings("unchecked")
    public <T> T claimsValue(String token, String key)
    {
        /* 如果token已存在缓存中就不需要去解析 */
        if (tokenCache.contains(token)) {
            Map<String, Object> claims = tokenCache.get(token);
            return (T) claims.get(key);
        }

        JwtParser jwtParser = Jwts.parser();

        /* 获取token使用的哪个配置 */
        TokenAlgorithmDecl tokenAlgorithmDecl = tokenAlgorithmDeclMap.get(JwtUtils.payload(token, _ENV));
        SignatureAlgorithm algorithm = tokenAlgorithmDecl.getAlgorithm();

        /* 使用不同的算法解析token */
        if (algorithm == SignatureAlgorithm.RS256)
            jwtParser.setSigningKey(DoubleCoder.PUBLICKEYRSA256(tokenAlgorithmDecl.getPublicKey()));

        else if (algorithm == SignatureAlgorithm.ES256)
            jwtParser.setSigningKey(DoubleCoder.PUBLICKEYEC256(tokenAlgorithmDecl.getPublicKey()));

        else if (algorithm == SignatureAlgorithm.HS256)
            jwtParser.setSigningKey(tokenAlgorithmDecl.getSecret());

        Claims claims = jwtParser.parseClaimsJws(token).getBody();
        return (T) claims.get(key);
    }

    @Override
    public Date getExpireTime(String token)
    {
        if (tokenCache.contains(token))
            return tokenCache.getExpire(token);

        return DateUtils.parse(claimsValue(token, EXPIRE_TIME));
    }

    @Override
    public boolean validate(String token)
    {
        try {
            if (tokenCache.contains(token))
                return true;

            claimsValue(token, "_");
            return true;
        }catch (Exception e){
            return false;
        }
    }

    @Override
    public Date flushExpireTime(String token)
    {
        validate(token);
        Date date = getExpireTime(token);
        return tokenCache.flushExpireTime(token, TimeUnits.SECONDS.add(date, currentTokenAlgorithmDecl.getExpireTime()));
    }
}
