package com.beifan.foxlibc.framework.utils;

import com.beifan.foxlibc.framework.utils.collection.Lists;
import com.beifan.foxlibc.framework.utils.collection.Maps;
import com.beifan.foxlibc.framework.utils.time.DateUtils;
import com.beifan.foxlibc.framework.utils.time.TimeUnits;
import lombok.AllArgsConstructor;

import java.util.*;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 本地缓存
 *
 * @author lts
 * Create Time 2022/7/1
 */
public class JvmCache {

    /**
     * 存放缓存的Map集合
     */
    private final Map<String, DelayValue> cacheValue = Maps.newHashMap();

    /**
     * 定时任务线程池
     */
    @SuppressWarnings("FieldCanBeLocal")
    private final ScheduledExecutorService executor = Executors.newSingleThreadScheduledExecutor();

    /**
     * 记录一次扫描扫描到已过期的KEY
     */
    private final List<String> expiredKeys = Lists.newArrayList();

    public JvmCache()
    {
        /* 每隔6秒钟扫描一次缓存, 更新掉过期的KEY */
        executor.scheduleAtFixedRate(()->{
            if (!cacheValue.isEmpty()) {
                for (Map.Entry<String, DelayValue> entry : cacheValue.entrySet()) {
                    DelayValue value = entry.getValue();
                    if (value.expired())
                        expiredKeys.add(entry.getKey());
                }

                if (!expiredKeys.isEmpty()) {
                    for (String expiredKey : expiredKeys) {
                        cacheValue.remove(expiredKey);
                    }
                    expiredKeys.clear();
                }
            }
        }, 0, 1000, TimeUnit.MILLISECONDS);
    }

    @AllArgsConstructor
    static class DelayValue {
        private Object value;
        private Date expireTime;
        public boolean expired()
        {
            return DateUtils.gteq(new Date(), expireTime);
        }

    }

    /**
     * 设置缓存
     *
     * @param key           缓存Key
     * @param value         缓存Value
     * @param expireTime    过期时间（单位秒）
     */
    public void set(String key, Object value, int expireTime)
    {
        set(key, value, TimeUnits.SECONDS.add(new Date(), expireTime));
    }

    /**
     * 设置缓存
     *
     * @param key           缓存Key
     * @param value         缓存Value
     * @param expireTime    过期时间
     */
    public void set(String key, Object value, Date expireTime)
    {
        Assert.throwIfNull(key, "缓存KEY不能为空");
        Assert.throwIfNull(value, "缓存VALUE不能为空");
        cacheValue.put(key, new DelayValue(value, expireTime));
    }

    /**
     * 获取缓存
     * @param key 缓存key
     */
    @SuppressWarnings("unchecked")
    public <T> T get(String key) {
        if (!cacheValue.containsKey(key))
            return null;

        /* GET的时候判断一次KEY是否过期 */
        DelayValue delayValue = cacheValue.get(key);
        if (DateUtils.gteq(new Date(), delayValue.expireTime)) {
            cacheValue.remove(key);
            return null;
        }

        return (T) delayValue.value;
    }

    /**
     * 删除缓存
     */
    public void remove(String key) {
        if (!cacheValue.containsKey(key))
            return;

        cacheValue.remove(key);
    }

    /**
     * 判断缓存是否存在
     */
    public boolean contains(String key)
    {
        return cacheValue.containsKey(key);
    }

    /**
     * 获取过期时间
     */
    public Date getExpire(String token)
    {
        DelayValue delayValue = cacheValue.get(token);
        return delayValue.expireTime;
    }

    /**
     * 刷新过期时间
     */
    public Date flushExpireTime(String key, Date flushExpireTime)
    {
        DelayValue delayValue = cacheValue.get(key);
        delayValue.expireTime = flushExpireTime;
        return delayValue.expireTime;
    }

    /**
     * 根据前缀查找
     */
    public <T> List<T> startWith(String startWith)
    {
        final List<String> keys = new ArrayList<>();

        cacheValue.forEach((key, value) -> {
            if (key.startsWith(startWith))
                keys.add(key);
        });

        List<T> values = Lists.newArrayList();
        keys.forEach(key -> values.add(this.get(key)));

        return values.stream().filter(Objects::nonNull).collect(Collectors.toList());
    }

}
