package com.beifan.foxlibc.framework.utils.config;

import com.beifan.foxlibc.framework.utils.Assert;
import com.beifan.foxlibc.framework.spring.service.jwt.JwtAlgorithm;
import com.beifan.foxlibc.framework.utils.collection.Maps;
import com.beifan.foxlibc.framework.utils.io.Files;
import io.jsonwebtoken.SignatureAlgorithm;
import lombok.Data;
import org.jdom2.Attribute;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.input.SAXBuilder;

import java.util.List;
import java.util.Map;

/**
 * 读取 __CODEINT 配置文件
 *
 * @author lts
 * Create Time 2022/6/30
 */
@Data
public class TokenAlgorithmDecl {

    private String name;
    private JwtAlgorithm algorithm;
    private String secret;
    private String publicKey;
    private String privateKey;
    private Integer expireTime = 604800; /* 单位为秒，默认七天 */

    public TokenAlgorithmDecl()
    {
    }

    public TokenAlgorithmDecl(String name, JwtAlgorithm algorithm, Integer expireTime)
    {
        this.name = name;
        this.algorithm = algorithm;

        if (expireTime != null && expireTime > 0)
            this.expireTime = expireTime;
    }

    public SignatureAlgorithm getAlgorithm()
    {
        return SignatureAlgorithm.forName(algorithm.name());
    }

    /**
     * 加载token配置文件
     */
    public static Map<String, TokenAlgorithmDecl> loadTokenAlgorithmDecls(String path)
    {
        try {
            Map<String, TokenAlgorithmDecl> jwtiniDeclMap = Maps.newHashMap();

            SAXBuilder builder = new SAXBuilder();
            Document doc = builder.build(Files.fromResource(path).getInputStream());
            Element element = doc.getRootElement();

            /* 获取算法列表 */
            List<Element> signatures = element.getChild("SIGNATURES").getChildren();
            for (Element signature : signatures) {
                Attribute attrEnv = signature.getAttribute("ENV");
                Attribute attrAlg = signature.getAttribute("ALG");
                Attribute attrExpireTime = signature.getAttribute("EXPTIME");

                /* 创建JWT算法声明 */
                TokenAlgorithmDecl tokenAlgorithmDecl = new TokenAlgorithmDecl(attrEnv.getValue(),
                        JwtAlgorithm.forName(attrAlg.getValue()), Integer.valueOf(attrExpireTime.getValue()));

                switch (tokenAlgorithmDecl.algorithm) {
                    case RS256:
                    case ES256: {
                        Element pubkey = signature.getChild("PUBKEY");
                        Element privkey = signature.getChild("PRIVKEY");
                        tokenAlgorithmDecl.setPublicKey(pubkey.getText());
                        tokenAlgorithmDecl.setPrivateKey(privkey.getText());
                        jwtiniDeclMap.put(tokenAlgorithmDecl.getName(), tokenAlgorithmDecl);
                        break;
                    }

                    case HS256: {
                        Element secret = signature.getChild("SECRET");
                        tokenAlgorithmDecl.setSecret(secret.getText());
                        jwtiniDeclMap.put(tokenAlgorithmDecl.getName(), tokenAlgorithmDecl);
                        break;
                    }
                }
            }

            return jwtiniDeclMap;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

}
